@@ -24,15 +24,15 @@ import static org.mockito.Mockito.spy;
 import static org.testng.Assert.assertEquals;
 
 import com.google.common.collect.Range;
-
 import java.lang.reflect.Method;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.NavigableMap;
 import java.util.concurrent.ConcurrentSkipListMap;
-
 import org.apache.bookkeeper.client.BookKeeper;
 import org.apache.bookkeeper.mledger.ManagedLedgerConfig;
+import org.apache.bookkeeper.mledger.Position;
+import org.apache.bookkeeper.mledger.PositionFactory;
 import org.apache.bookkeeper.mledger.proto.MLDataFormats.ManagedLedgerInfo.LedgerInfo;
 import org.apache.bookkeeper.mledger.proto.MLDataFormats.MessageRange;
 import org.apache.bookkeeper.mledger.proto.MLDataFormats.NestedPositionInfo;
@@ -59,7 +59,7 @@ public class ManagedCursorIndividualDeletedMessagesTest {
         doReturn(config).when(ledger).getConfig();
 
         ManagedCursorImpl cursor = spy(new ManagedCursorImpl(bookkeeper, ledger, "test-cursor"));
-        LongPairRangeSet<PositionImpl> deletedMessages = cursor.getIndividuallyDeletedMessagesSet();
+        LongPairRangeSet<Position> deletedMessages = cursor.getIndividuallyDeletedMessagesSet();
 
         Method recoverMethod = ManagedCursorImpl.class.getDeclaredMethod("recoverIndividualDeletedMessages",
                 List.class);
@@ -68,7 +68,7 @@ public class ManagedCursorIndividualDeletedMessagesTest {
         // (1) [(1:5..1:10]]
         List<MessageRange> messageRangeList = new ArrayList();
         messageRangeList.add(createMessageRange(1, 5, 1, 10));
-        List<Range<PositionImpl>> expectedRangeList = new ArrayList();
+        List<Range<Position>> expectedRangeList = new ArrayList();
         expectedRangeList.add(createPositionRange(1, 5, 1, 10));
         recoverMethod.invoke(cursor, messageRangeList);
         assertEquals(deletedMessages.size(), 1);
@@ -120,9 +120,9 @@ public class ManagedCursorIndividualDeletedMessagesTest {
         return messageRangeBuilder.build();
     }
 
-    private static Range<PositionImpl> createPositionRange(long lowerLedgerId, long lowerEntryId, long upperLedgerId,
+    private static Range<Position> createPositionRange(long lowerLedgerId, long lowerEntryId, long upperLedgerId,
             long upperEntryId) {
-        return Range.openClosed(new PositionImpl(lowerLedgerId, lowerEntryId),
-                new PositionImpl(upperLedgerId, upperEntryId));
+        return Range.openClosed(PositionFactory.create(lowerLedgerId, lowerEntryId),
+                PositionFactory.create(upperLedgerId, upperEntryId));
     }
 }
