@@ -41,7 +41,7 @@ import org.slf4j.LoggerFactory;
  * configured threshold. As a consequence, this strategy tends to distribute load among all brokers. It does this by
  * first computing the average resource usage per broker for the whole cluster. The resource usage for each broker is
  * calculated using the following method:
- * {@link LocalBrokerData#getMaxResourceUsageWithWeight(double, double, double, double, double)}. The weights
+ * {@link LocalBrokerData#getMaxResourceUsageWithWeight(double, double, double, double)}. The weights
  * for each resource are configurable. Historical observations are included in the running average based on the broker's
  * setting for loadBalancerHistoryResourcePercentage. Once the average resource usage is calculated, a broker's
  * current/historical usage is compared to the average broker usage. If a broker's usage is greater than the average
@@ -119,7 +119,7 @@ public class ThresholdShedder implements LoadSheddingStrategy {
             }
         });
         if (selectedBundlesCache.isEmpty() && conf.isLowerBoundarySheddingEnabled()) {
-            tryLowerBoundaryShedding(loadData, threshold, conf);
+            tryLowerBoundaryShedding(loadData, conf);
         }
         return selectedBundlesCache;
     }
@@ -173,8 +173,8 @@ public class ThresholdShedder implements LoadSheddingStrategy {
         double resourceUsage = localBrokerData.getMaxResourceUsageWithWeight(
                 conf.getLoadBalancerCPUResourceWeight(),
                 conf.getLoadBalancerDirectMemoryResourceWeight(),
-                conf.getLoadBalancerBandwithInResourceWeight(),
-                conf.getLoadBalancerBandwithOutResourceWeight());
+                conf.getLoadBalancerBandwidthInResourceWeight(),
+                conf.getLoadBalancerBandwidthOutResourceWeight());
         historyUsage = historyUsage == null
                 ? resourceUsage : historyUsage * historyPercentage + (1 - historyPercentage) * resourceUsage;
 
@@ -182,12 +182,18 @@ public class ThresholdShedder implements LoadSheddingStrategy {
         return historyUsage;
     }
 
-    private void tryLowerBoundaryShedding(LoadData loadData, double threshold, ServiceConfiguration conf) {
+    private void tryLowerBoundaryShedding(LoadData loadData, ServiceConfiguration conf) {
         // Select the broker with the most resource usage.
+        final double threshold = conf.getLoadBalancerBrokerThresholdShedderPercentage() / 100.0;
         final double avgUsage = getBrokerAvgUsage(loadData, conf.getLoadBalancerHistoryResourcePercentage(), conf);
         Pair<Boolean, String> result = getMaxUsageBroker(loadData, threshold, avgUsage);
         boolean hasBrokerBelowLowerBound = result.getLeft();
         String maxUsageBroker = result.getRight();
+        BrokerData brokerData = loadData.getBrokerData().get(maxUsageBroker);
+        if (brokerData == null) {
+            log.info("Load data is null or bundle <=1, skipping bundle unload.");
+            return;
+        }
         if (!hasBrokerBelowLowerBound) {
             log.info("No broker is below the lower bound, threshold is {}, "
                             + "avgUsage usage is {}, max usage of Broker {} is {}",
@@ -195,11 +201,6 @@ public class ThresholdShedder implements LoadSheddingStrategy {
                     brokerAvgResourceUsage.getOrDefault(maxUsageBroker, 0.0));
             return;
         }
-        BrokerData brokerData = loadData.getBrokerData().get(maxUsageBroker);
-        if (brokerData == null) {
-            log.info("Load data is null or bundle <=1, skipping bundle unload.");
-            return;
-        }
         LocalBrokerData localData = brokerData.getLocalData();
         double brokerCurrentThroughput = localData.getMsgThroughputIn() + localData.getMsgThroughputOut();
         double minimumThroughputToOffload = brokerCurrentThroughput * threshold * LOWER_BOUNDARY_THRESHOLD_MARGIN;
