@@ -27,11 +27,13 @@ public interface Replicator {
 
     void startProducer();
 
-    ReplicatorStatsImpl getStats();
+    Topic getLocalTopic();
+
+    ReplicatorStatsImpl computeStats();
 
     CompletableFuture<Void> terminate();
 
-    CompletableFuture<Void> disconnect();
+    CompletableFuture<Void> disconnect(boolean failIfHasBacklog, boolean closeTheStartingProducer);
 
     void updateRates();
 
@@ -53,4 +55,6 @@ public interface Replicator {
     long getNumberOfEntriesInBacklog();
 
     boolean isTerminated();
+
+    ReplicatorStatsImpl getStats();
 }
