@@ -19,7 +19,6 @@
 package org.apache.pulsar.broker.service;
 
 import static org.assertj.core.api.Assertions.assertThat;
-import static org.assertj.core.api.Assertions.assertThatThrownBy;
 import static org.testng.Assert.assertEquals;
 import static org.testng.Assert.assertFalse;
 import static org.testng.Assert.assertNotNull;
@@ -30,7 +29,6 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
-import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
@@ -66,7 +64,6 @@ import org.apache.pulsar.client.impl.BatchMessageIdImpl;
 import org.apache.pulsar.client.impl.ClientBuilderImpl;
 import org.apache.pulsar.client.impl.ClientCnx;
 import org.apache.pulsar.client.impl.MessageIdImpl;
-import org.apache.pulsar.client.impl.TopicMessageIdImpl;
 import org.apache.pulsar.client.impl.metrics.InstrumentProvider;
 import org.apache.pulsar.common.api.proto.CommandError;
 import org.apache.pulsar.common.naming.TopicName;
@@ -89,7 +86,6 @@ public class SubscriptionSeekTest extends BrokerTestBase {
         conf.setManagedLedgerMaxEntriesPerLedger(10);
         conf.setDefaultRetentionSizeInMB(100);
         conf.setDefaultRetentionTimeInMinutes(100);
-        conf.setAutoSkipNonRecoverableData(true);
         super.baseSetup();
         conf.setAcknowledgmentAtBatchIndexLevelEnabled(true);
     }
@@ -469,22 +465,6 @@ public class SubscriptionSeekTest extends BrokerTestBase {
         }
     }
 
-    @Test
-    public void testSeekWithNonOwnerTopicMessage() throws Exception {
-        final String topicName = "persistent://prop/use/ns-abc/testNonOwnerTopicMessage";
-
-        admin.topics().createPartitionedTopic(topicName, 2);
-        @Cleanup
-        org.apache.pulsar.client.api.Consumer<byte[]> consumer = pulsarClient.newConsumer().topic(topicName)
-                .subscriptionName("my-subscription").subscribe();
-        assertThatThrownBy(
-                // seek with a TopicMessageIdImpl that has a null topic.
-                () -> consumer.seek(new TopicMessageIdImpl(null, new BatchMessageIdImpl(123L, 345L, 566, 789)))
-            )
-            .isInstanceOf(PulsarClientException.class)
-            .hasMessage("The owner topic is null");
-    }
-
     @Test
     public void testSeekTime() throws Exception {
         final String topicName = "persistent://prop/use/ns-abc/testSeekTime";
@@ -566,90 +546,6 @@ public class SubscriptionSeekTest extends BrokerTestBase {
         }
     }
 
-    @Test(timeOut = 30_000)
-    public void testSeekByTimestampWithSkipNonRecoverableData() throws Exception {
-        String topicName = "persistent://prop/use/ns-abc/testSeekByTimestampWithSkipNonRecoverableData";
-        admin.topics().createNonPartitionedTopic(topicName);
-        admin.topics().createSubscription(topicName, "my-sub", MessageId.earliest);
-
-        @Cleanup
-        Producer<String> producer =
-            pulsarClient.newProducer(Schema.STRING).topic(topicName).enableBatching(false).create();
-        for (int i = 0; i < 55; i++) {
-            producer.send(("message-" + i));
-            Thread.sleep(10);
-        }
-
-        Map<Long, MessageIdImpl> timestampToMessageId = new HashMap<>();
-        List<Long> ledgerIds = new ArrayList<>();
-        @Cleanup
-        Reader<String> reader =
-            pulsarClient.newReader(Schema.STRING).topic(topicName).startMessageId(MessageId.earliest).create();
-        while (reader.hasMessageAvailable()) {
-            Message<String> message = reader.readNext();
-            log.info("message: {} ----- {}", message.getMessageId(), message.getPublishTime());
-            timestampToMessageId.put(message.getPublishTime(), (MessageIdImpl) message.getMessageId());
-            long ledgerId = ((MessageIdImpl) message.getMessageId()).getLedgerId();
-            if (!ledgerIds.contains(ledgerId)) {
-                ledgerIds.add(ledgerId);
-            }
-        }
-
-        Assert.assertEquals(timestampToMessageId.size(), 55);
-
-        LinkedHashSet<Long> deletedLedgerIds = new LinkedHashSet<>();
-        deletedLedgerIds.add(ledgerIds.get(0));
-        deletedLedgerIds.add(ledgerIds.get(ledgerIds.size() - 1));
-        int mid = ledgerIds.size() / 2;
-        deletedLedgerIds.add(ledgerIds.get(mid));
-
-        for (Long deletedLedgerId : deletedLedgerIds) {
-            pulsar.getBookKeeperClient().deleteLedger(deletedLedgerId);
-            log.info("delete ledger: {}", deletedLedgerId);
-        }
-
-        admin.topics().unload(topicName);
-
-        @Cleanup
-        org.apache.pulsar.client.api.Consumer<String> consumer = pulsarClient.newConsumer(Schema.STRING)
-            .receiverQueueSize(0)
-            .topic(topicName).subscriptionName("my-sub")
-            .subscriptionInitialPosition(SubscriptionInitialPosition.Earliest).subscribe();
-        long[] timestamps = timestampToMessageId.keySet().stream().mapToLong(Long::longValue).toArray();
-
-        Arrays.sort(timestamps);
-
-        timestampToMessageId.values().removeIf(messageId -> deletedLedgerIds.contains(messageId.getLedgerId()));
-
-        final int lastNonRecoverableEntryNums = 5;
-
-        for (int i = 0; i < timestamps.length - lastNonRecoverableEntryNums; i++) {
-            MessageIdImpl nextValidMessageId = timestampToMessageId.get(timestamps[i]);
-            int l = i;
-            while (nextValidMessageId == null) {
-                nextValidMessageId = timestampToMessageId.get(timestamps[l++]);
-            }
-
-            consumer.seek(timestamps[i]);
-            Message<String> receive = consumer.receive();
-
-            MessageIdImpl msgId = (MessageIdImpl) receive.getMessageId();
-            Assert.assertEquals(msgId.getLedgerId(), nextValidMessageId.getLedgerId());
-            Assert.assertEquals(msgId.getEntryId(), nextValidMessageId.getEntryId());
-        }
-
-        MessageIdImpl lastMessageId = (MessageIdImpl) producer.send(("message-last"));
-
-        for (int i = timestamps.length - lastNonRecoverableEntryNums; i < timestamps.length; i++) {
-            consumer.seek(timestamps[i]);
-            Message<String> receive = consumer.receive();
-
-            MessageIdImpl msgId = (MessageIdImpl) receive.getMessageId();
-            Assert.assertEquals(msgId.getLedgerId(), lastMessageId.getLedgerId());
-            Assert.assertEquals(msgId.getEntryId(), lastMessageId.getEntryId());
-        }
-    }
-
     @Test(timeOut = 30_000)
     public void testSeekByTimestampWithLedgerTrim() throws Exception {
         String topicName = "persistent://prop/use/ns-abc/testSeekByTimestampWithLedgerTrim";
@@ -1032,7 +928,7 @@ public class SubscriptionSeekTest extends BrokerTestBase {
     public void testSeekWillNotEncounteredFencedError() throws Exception {
         String topicName = "persistent://prop/ns-abc/my-topic2";
         admin.topics().createNonPartitionedTopic(topicName);
-        admin.topicPolicies().setRetention(topicName, new RetentionPolicies(3600, -1));
+        admin.topicPolicies().setRetention(topicName, new RetentionPolicies(3600, 0));
         // Create a pulsar client with a subscription fenced counter.
         ClientBuilderImpl clientBuilder = (ClientBuilderImpl) PulsarClient.builder().serviceUrl(lookupUrl.toString());
         AtomicInteger receivedFencedErrorCounter = new AtomicInteger();
