@@ -46,8 +46,6 @@ import lombok.Cleanup;
 import lombok.extern.slf4j.Slf4j;
 import org.apache.bookkeeper.mledger.Position;
 import org.apache.bookkeeper.mledger.impl.ManagedLedgerImpl;
-import org.apache.bookkeeper.mledger.impl.PositionImpl;
-import org.apache.pulsar.broker.BrokerTestUtil;
 import org.apache.pulsar.broker.TransactionMetadataStoreService;
 import org.apache.pulsar.broker.service.BrokerService;
 import org.apache.pulsar.broker.service.Topic;
@@ -75,7 +73,6 @@ import org.apache.pulsar.client.util.RetryMessageUtil;
 import org.apache.pulsar.common.api.proto.CommandAck;
 import org.apache.pulsar.common.naming.TopicName;
 import org.apache.pulsar.common.policies.data.PersistentTopicInternalStats;
-import org.apache.pulsar.common.util.collections.ConcurrentOpenHashMap;
 import org.apache.pulsar.transaction.coordinator.TransactionCoordinatorID;
 import org.apache.pulsar.transaction.coordinator.TransactionMetadataStore;
 import org.apache.pulsar.transaction.coordinator.TransactionSubscription;
@@ -417,11 +414,7 @@ public class TransactionEndToEndTest extends TransactionTestBase {
                 boolean exist = false;
                 for (int i = 0; i < getPulsarServiceList().size(); i++) {
 
-                    Field field = BrokerService.class.getDeclaredField("topics");
-                    field.setAccessible(true);
-                    ConcurrentOpenHashMap<String, CompletableFuture<Optional<Topic>>> topics =
-                            (ConcurrentOpenHashMap<String, CompletableFuture<Optional<Topic>>>) field
-                                    .get(getPulsarServiceList().get(i).getBrokerService());
+                    final var topics = getPulsarServiceList().get(i).getBrokerService().getTopics();
                     CompletableFuture<Optional<Topic>> topicFuture = topics.get(topic);
 
                     if (topicFuture != null) {
@@ -440,7 +433,7 @@ public class TransactionEndToEndTest extends TransactionTestBase {
                                 //when delete commit marker operation finish, it can run next delete commit marker operation
                                 //so this test may not delete all the position in this manageLedger.
                                 Position markerPosition = ((ManagedLedgerImpl) persistentSubscription.getCursor()
-                                        .getManagedLedger()).getNextValidPosition((PositionImpl) markDeletePosition);
+                                        .getManagedLedger()).getNextValidPosition(markDeletePosition);
                                 //marker is the lastConfirmedEntry, after commit the marker will only be write in
                                 if (!markerPosition.equals(lastConfirmedEntry)) {
                                     log.error("Mark delete position is not commit marker position!");
@@ -462,76 +455,8 @@ public class TransactionEndToEndTest extends TransactionTestBase {
         log.info("finished test partitionAbortTest");
     }
 
-    @DataProvider
-    public Object[][] unackMessagesCountParams() {
-        return new Object[][] {
-                // batchSend, batchAck, asyncAck
-                {Boolean.TRUE, Boolean.TRUE, Boolean.TRUE},
-                {Boolean.TRUE, Boolean.TRUE, Boolean.FALSE},
-                {Boolean.TRUE, Boolean.FALSE, Boolean.TRUE},
-                {Boolean.TRUE, Boolean.FALSE, Boolean.FALSE},
-                {Boolean.FALSE, Boolean.TRUE, Boolean.TRUE},
-                {Boolean.FALSE, Boolean.TRUE, Boolean.FALSE},
-                {Boolean.FALSE, Boolean.FALSE, Boolean.TRUE},
-                {Boolean.FALSE, Boolean.FALSE, Boolean.FALSE}
-        };
-    }
-
-    @Test(dataProvider= "unackMessagesCountParams")
-    public void testUnackMessageAfterAckAllMessages(boolean batchSend, boolean batchAck, boolean asyncAck)
-            throws Exception {
-        final int messageCount = 50;
-        final String subName = "s1";
-        final String topicName = BrokerTestUtil.newUniqueName(NAMESPACE1 + "/tp");
-        // Create producer and consumer.
-        Consumer<byte[]> consumer = pulsarClient.newConsumer().topic(topicName).subscriptionName(subName)
-                .subscriptionType(SubscriptionType.Shared).isAckReceiptEnabled(true).subscribe();
-        Awaitility.await().until(consumer::isConnected);
-        Producer<byte[]> producer = pulsarClient.newProducer().topic(topicName).enableBatching(batchSend)
-                .batchingMaxMessages(10).create();
-        Awaitility.await().until(producer::isConnected);
-        // Publish messages.
-        CompletableFuture<MessageId> latestSendFuture = null;
-        for (int i = 0; i < messageCount; i++) {
-            latestSendFuture = producer.sendAsync((i + "").getBytes());
-        }
-        latestSendFuture.join();
-
-        // Ack messages with TXN.
-        Transaction txn = getTxn();
-        List<MessageId> msgIds = new ArrayList<>();
-        for (int i = 0; i < messageCount; i++) {
-            Message<byte[]> message = consumer.receive(waitTimeForCanReceiveMsgInSec, TimeUnit.SECONDS);
-            msgIds.add(message.getMessageId());
-        }
-        if (batchAck) {
-            consumer.acknowledgeAsync(msgIds, txn).get();
-        } else if (asyncAck) {
-            CompletableFuture<Void> latestAckFuture = null;
-            for (MessageId msgId : msgIds) {
-                latestAckFuture = consumer.acknowledgeAsync(msgId, txn);
-            }
-            latestAckFuture.join();
-        } else {
-            for (MessageId msgId : msgIds) {
-                consumer.acknowledgeAsync(msgId, txn).get();
-            }
-        }
-        txn.commit().get();
-
-        // Verify: the quantity of un-ack messages is 0.
-        int unAckMsgs = admin.topics().getStats(topicName).getSubscriptions().get(subName).getConsumers().get(0)
-                .getUnackedMessages();
-        assertEquals(unAckMsgs, 0);
-
-        // cleanup.
-        producer.close();
-        consumer.close();
-        admin.topics().delete(topicName, true);
-    }
-
     @Test(dataProvider="enableBatch")
-    public void testAckWithTransactionReduceUnAckMessageCount(boolean enableBatch) throws Exception {
+    private void testAckWithTransactionReduceUnAckMessageCount(boolean enableBatch) throws Exception {
 
         final int messageCount = 50;
         final String subName = "testAckWithTransactionReduceUnAckMessageCount";
@@ -792,9 +717,7 @@ public class TransactionEndToEndTest extends TransactionTestBase {
 
             Field field = BrokerService.class.getDeclaredField("topics");
             field.setAccessible(true);
-            ConcurrentOpenHashMap<String, CompletableFuture<Optional<Topic>>> topics =
-                    (ConcurrentOpenHashMap<String, CompletableFuture<Optional<Topic>>>) field
-                            .get(getPulsarServiceList().get(i).getBrokerService());
+            final var topics = getPulsarServiceList().get(i).getBrokerService().getTopics();
             CompletableFuture<Optional<Topic>> topicFuture = topics.get(topic);
 
             if (topicFuture != null) {
@@ -813,7 +736,7 @@ public class TransactionEndToEndTest extends TransactionTestBase {
                         //when delete commit marker operation finish, it can run next delete commit marker operation
                         //so this test may not delete all the position in this manageLedger.
                         Position markerPosition = ((ManagedLedgerImpl) persistentSubscription.getCursor()
-                                .getManagedLedger()).getNextValidPosition((PositionImpl) markDeletePosition);
+                                .getManagedLedger()).getNextValidPosition(markDeletePosition);
                         //marker is the lastConfirmedEntry, after commit the marker will only be write in
                         if (!markerPosition.equals(lastConfirmedEntry)) {
                             log.error("Mark delete position is not commit marker position!");
@@ -1263,9 +1186,7 @@ public class TransactionEndToEndTest extends TransactionTestBase {
 
             Field field = BrokerService.class.getDeclaredField("topics");
             field.setAccessible(true);
-            ConcurrentOpenHashMap<String, CompletableFuture<Optional<Topic>>> topics =
-                    (ConcurrentOpenHashMap<String, CompletableFuture<Optional<Topic>>>) field
-                            .get(getPulsarServiceList().get(i).getBrokerService());
+            final var topics = getPulsarServiceList().get(i).getBrokerService().getTopics();
             CompletableFuture<Optional<Topic>> topicFuture = topics.get(topic);
 
             if (topicFuture != null) {
@@ -1579,7 +1500,8 @@ public class TransactionEndToEndTest extends TransactionTestBase {
 
         @Cleanup
         Consumer<byte[]> deadLetterConsumer = pulsarClient.newConsumer()
-                .topic(RetryMessageUtil.getDLQTopic(topic, subName))
+                .topic(String.format("%s-%s" + RetryMessageUtil.DLQ_GROUP_TOPIC_SUFFIX,
+                        topic, subName))
                 .subscriptionType(SubscriptionType.Shared)
                 .deadLetterPolicy(DeadLetterPolicy.builder().maxRedeliverCount(1).build())
                 .subscriptionName("test")
@@ -1614,7 +1536,8 @@ public class TransactionEndToEndTest extends TransactionTestBase {
         consumer.close();
         deadLetterConsumer.close();
         producer.close();
-        admin.topics().delete(RetryMessageUtil.getDLQTopic(topic, subName), true);
+        admin.topics().delete(String.format("%s-%s" + RetryMessageUtil.DLQ_GROUP_TOPIC_SUFFIX,
+                topic, subName), true);
         admin.topics().delete(topic, true);
     }
 
@@ -1641,7 +1564,8 @@ public class TransactionEndToEndTest extends TransactionTestBase {
 
         @Cleanup
         Consumer<byte[]> deadLetterConsumer = pulsarClient.newConsumer()
-                .topic(RetryMessageUtil.getDLQTopic(topic, subName))
+                .topic(String.format("%s-%s" + RetryMessageUtil.DLQ_GROUP_TOPIC_SUFFIX,
+                        topic, subName))
                 .subscriptionType(SubscriptionType.Shared)
                 .deadLetterPolicy(DeadLetterPolicy.builder().maxRedeliverCount(1).build())
                 .subscriptionName("test")
@@ -1687,7 +1611,8 @@ public class TransactionEndToEndTest extends TransactionTestBase {
         consumer.close();
         deadLetterConsumer.close();
         producer.close();
-        admin.topics().delete(RetryMessageUtil.getDLQTopic(topic, subName), true);
+        admin.topics().delete(String.format("%s-%s" + RetryMessageUtil.DLQ_GROUP_TOPIC_SUFFIX,
+                topic, subName), true);
         admin.topics().delete(topic, true);
     }
 
@@ -1720,24 +1645,24 @@ public class TransactionEndToEndTest extends TransactionTestBase {
         for (int i = 0; i < 10; i++) {
             producer.newMessage(transaction)
                     .value("msg-" + i)
-                    .deliverAfter(5, TimeUnit.SECONDS)
+                    .deliverAfter(7, TimeUnit.SECONDS)
                     .sendAsync();
         }
 
         producer.flush();
 
         transaction.commit().get();
-
-        // Failover consumer will receive the messages immediately while
-        // the shared consumer will get them after the delay
-        Message<String> msg = sharedConsumer.receive(waitTimeForCannotReceiveMsgInSec, TimeUnit.SECONDS);
-        assertNull(msg);
-
+        Message<String> msg;
         for (int i = 0; i < 10; i++) {
             msg = failoverConsumer.receive(waitTimeForCanReceiveMsgInSec, TimeUnit.SECONDS);
             assertEquals(msg.getValue(), "msg-" + i);
         }
 
+        // Failover consumer will receive the messages immediately while
+        // the shared consumer will get them after the delay
+        msg = sharedConsumer.receive(waitTimeForCannotReceiveMsgInSec, TimeUnit.SECONDS);
+        assertNull(msg);
+
         Set<String> receivedMsgs = new TreeSet<>();
         for (int i = 0; i < 10; i++) {
             msg = sharedConsumer.receive(waitTimeForCanReceiveMsgInSec, TimeUnit.SECONDS);
