@@ -26,8 +26,9 @@ import java.util.concurrent.atomic.AtomicReference;
 import org.apache.bookkeeper.mledger.AsyncCallbacks;
 import org.apache.bookkeeper.mledger.Entry;
 import org.apache.bookkeeper.mledger.ManagedLedgerException;
+import org.apache.bookkeeper.mledger.Position;
+import org.apache.bookkeeper.mledger.PositionFactory;
 import org.apache.bookkeeper.mledger.impl.ManagedCursorImpl;
-import org.apache.bookkeeper.mledger.impl.PositionImpl;
 import org.mockito.Mockito;
 import org.testng.Assert;
 import org.testng.annotations.Test;
@@ -36,7 +37,7 @@ public class CompactedTopicUtilsTest {
 
     @Test
     public void testReadCompactedEntriesWithEmptyEntries() throws ExecutionException, InterruptedException {
-        PositionImpl lastCompactedPosition = PositionImpl.get(1, 100);
+        Position lastCompactedPosition = PositionFactory.create(1, 100);
         TopicCompactionService service = Mockito.mock(TopicCompactionService.class);
         Mockito.doReturn(CompletableFuture.completedFuture(Collections.emptyList()))
                 .when(service).readCompactedEntries(Mockito.any(), Mockito.intThat(argument -> argument > 0));
@@ -44,8 +45,8 @@ public class CompactedTopicUtilsTest {
                 .getLastCompactedPosition();
 
 
-        PositionImpl initPosition = PositionImpl.get(1, 90);
-        AtomicReference<PositionImpl> readPositionRef = new AtomicReference<>(initPosition.getNext());
+        Position initPosition = PositionFactory.create(1, 90);
+        AtomicReference<Position> readPositionRef = new AtomicReference<>(initPosition.getNext());
         ManagedCursorImpl cursor = Mockito.mock(ManagedCursorImpl.class);
         Mockito.doReturn(readPositionRef.get()).when(cursor).getReadPosition();
         Mockito.doReturn(1).when(cursor).applyMaxSizeCap(Mockito.anyInt(), Mockito.anyLong());
@@ -69,8 +70,10 @@ public class CompactedTopicUtilsTest {
             }
         };
 
-        final var entries = CompactedTopicUtils.asyncReadCompactedEntries(service, cursor, 1, 100,
-                PositionImpl.LATEST, false, false).get();
+        CompactedTopicUtils.asyncReadCompactedEntries(service, cursor, 1, 100,
+                PositionFactory.LATEST, false, readEntriesCallback, false, null);
+
+        List<Entry> entries = completableFuture.get();
         Assert.assertTrue(entries.isEmpty());
         Assert.assertNull(throwableRef.get());
         Assert.assertEquals(readPositionRef.get(), lastCompactedPosition.getNext());
