@@ -47,7 +47,7 @@ import org.apache.bookkeeper.client.LedgerEntry;
 import org.apache.bookkeeper.client.LedgerHandle;
 import org.apache.bookkeeper.mledger.Entry;
 import org.apache.bookkeeper.mledger.Position;
-import org.apache.bookkeeper.mledger.impl.PositionImpl;
+import org.apache.bookkeeper.mledger.PositionFactory;
 import org.apache.pulsar.broker.BrokerTestUtil;
 import org.apache.pulsar.broker.ServiceConfiguration;
 import org.apache.pulsar.broker.auth.MockedPulsarServiceBaseTest;
@@ -101,7 +101,7 @@ public class CompactorTest extends MockedPulsarServiceBaseTest {
                 new ThreadFactoryBuilder().setNameFormat("compactor").setDaemon(true).build());
         bk = pulsar.getBookKeeperClientFactory().create(
                 this.conf, null, null, Optional.empty(), null).get();
-        compactor = new TwoPhaseCompactor(conf, pulsarClient, bk, compactionScheduler);
+        compactor = new PublishingOrderCompactor(conf, pulsarClient, bk, compactionScheduler);
     }
 
 
@@ -127,7 +127,7 @@ public class CompactorTest extends MockedPulsarServiceBaseTest {
         return compactor;
     }
 
-    private List<String> compactAndVerify(String topic, Map<String, byte[]> expected, boolean checkMetrics)
+    protected List<String> compactAndVerify(String topic, Map<String, byte[]> expected, boolean checkMetrics)
             throws Exception {
 
         long compactedLedgerId = compact(topic);
@@ -361,7 +361,7 @@ public class CompactorTest extends MockedPulsarServiceBaseTest {
         PulsarClientImpl mockClient = mock(PulsarClientImpl.class);
         ConnectionPool connectionPool = mock(ConnectionPool.class);
         when(mockClient.getCnxPool()).thenReturn(connectionPool);
-        TwoPhaseCompactor compactor = new TwoPhaseCompactor(configuration, mockClient,
+        PublishingOrderCompactor compactor = new PublishingOrderCompactor(configuration, mockClient,
                 Mockito.mock(BookKeeper.class), compactionScheduler);
         Assert.assertEquals(compactor.getPhaseOneLoopReadTimeoutInSeconds(), 60);
     }
@@ -402,7 +402,7 @@ public class CompactorTest extends MockedPulsarServiceBaseTest {
         Position lastCompactedPosition = topicCompactionService.getLastCompactedPosition().get();
         Entry lastCompactedEntry = topicCompactionService.readLastCompactedEntry().get();
 
-        Assert.assertTrue(PositionImpl.get(lastCompactedPosition.getLedgerId(), lastCompactedPosition.getEntryId())
+        Assert.assertTrue(PositionFactory.create(lastCompactedPosition.getLedgerId(), lastCompactedPosition.getEntryId())
                 .compareTo(lastCompactedEntry.getLedgerId(), lastCompactedEntry.getEntryId()) >= 0);
 
         future.join();
