@@ -18,14 +18,13 @@
  */
 package org.apache.pulsar.client.impl;
 
+import com.google.re2j.Pattern;
 import io.netty.channel.ChannelHandlerContext;
-import java.util.List;
 import java.util.concurrent.CompletableFuture;
-import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicInteger;
 import java.util.concurrent.atomic.AtomicLongFieldUpdater;
 import java.util.concurrent.atomic.AtomicReference;
-import java.util.regex.Pattern;
 import org.apache.pulsar.client.api.PulsarClientException;
 import org.apache.pulsar.common.api.proto.BaseCommand;
 import org.apache.pulsar.common.api.proto.CommandWatchTopicUpdate;
@@ -43,7 +42,7 @@ public class TopicListWatcher extends HandlerState implements ConnectionHandler.
             AtomicLongFieldUpdater
                     .newUpdater(TopicListWatcher.class, "createWatcherDeadline");
 
-    private final PatternMultiTopicsConsumerImpl.TopicsChangedListener topicsChangeListener;
+    private final PatternConsumerUpdateQueue patternConsumerUpdateQueue;
     private final String name;
     private final ConnectionHandler connectionHandler;
     private final Pattern topicsPattern;
@@ -54,7 +53,7 @@ public class TopicListWatcher extends HandlerState implements ConnectionHandler.
     private String topicsHash;
     private final CompletableFuture<TopicListWatcher> watcherFuture;
 
-    private final List<Throwable> previousExceptions = new CopyOnWriteArrayList<>();
+    private final AtomicInteger previousExceptionCount = new AtomicInteger();
     private final AtomicReference<ClientCnx> clientCnxUsedForWatcherRegistration = new AtomicReference<>();
 
     private final Runnable recheckTopicsChangeAfterReconnect;
@@ -63,13 +62,13 @@ public class TopicListWatcher extends HandlerState implements ConnectionHandler.
     /***
      * @param topicsPattern The regexp for the topic name(not contains partition suffix).
      */
-    public TopicListWatcher(PatternMultiTopicsConsumerImpl.TopicsChangedListener topicsChangeListener,
+    public TopicListWatcher(PatternConsumerUpdateQueue patternConsumerUpdateQueue,
                             PulsarClientImpl client, Pattern topicsPattern, long watcherId,
                             NamespaceName namespace, String topicsHash,
                             CompletableFuture<TopicListWatcher> watcherFuture,
                             Runnable recheckTopicsChangeAfterReconnect) {
         super(client, topicsPattern.pattern());
-        this.topicsChangeListener = topicsChangeListener;
+        this.patternConsumerUpdateQueue = patternConsumerUpdateQueue;
         this.name = "Watcher(" + topicsPattern + ")";
         this.connectionHandler = new ConnectionHandler(this,
                 new BackoffBuilder()
@@ -93,7 +92,7 @@ public class TopicListWatcher extends HandlerState implements ConnectionHandler.
     public void connectionFailed(PulsarClientException exception) {
         boolean nonRetriableError = !PulsarClientException.isRetriableError(exception);
         if (nonRetriableError) {
-            exception.setPreviousExceptions(previousExceptions);
+            exception.setPreviousExceptionCount(previousExceptionCount);
             if (watcherFuture.completeExceptionally(exception)) {
                 setState(State.Failed);
                 log.info("[{}] Watcher creation failed for {} with non-retriable error {}",
@@ -101,13 +100,13 @@ public class TopicListWatcher extends HandlerState implements ConnectionHandler.
                 deregisterFromClientCnx();
             }
         } else {
-            previousExceptions.add(exception);
+            previousExceptionCount.incrementAndGet();
         }
     }
 
     @Override
     public CompletableFuture<Void> connectionOpened(ClientCnx cnx) {
-        previousExceptions.clear();
+        previousExceptionCount.set(0);
 
         State state = getState();
         if (state == State.Closing || state == State.Closed) {
@@ -277,13 +276,7 @@ public class TopicListWatcher extends HandlerState implements ConnectionHandler.
     }
 
     public void handleCommandWatchTopicUpdate(CommandWatchTopicUpdate update) {
-        List<String> deleted = update.getDeletedTopicsList();
-        if (!deleted.isEmpty()) {
-            topicsChangeListener.onTopicsRemoved(deleted);
-        }
-        List<String> added = update.getNewTopicsList();
-        if (!added.isEmpty()) {
-            topicsChangeListener.onTopicsAdded(added);
-        }
+        patternConsumerUpdateQueue.appendTopicsRemovedOp(update.getDeletedTopicsList());
+        patternConsumerUpdateQueue.appendTopicsAddedOp(update.getNewTopicsList());
     }
 }
