@@ -98,26 +98,20 @@ public class PerformanceProducerTest extends MockedPulsarServiceBaseTest {
 
         thread.start();
 
-        int count1 = 0;
-        int count2 = 0;
-        for (int i = 0; i < 10; i++) {
-            Message<byte[]> message = consumer1.receive(1, TimeUnit.SECONDS);
-            if (message == null) {
-                break;
-            }
-            count1++;
-            consumer1.acknowledge(message);
-        }
-        for (int i = 0; i < 10; i++) {
-            Message<byte[]> message = consumer2.receive(1, TimeUnit.SECONDS);
-            if (message == null) {
-                break;
-            }
-            count2++;
-            consumer2.acknowledge(message);
-        }
-        //in key_share mode, only one consumer can get msg
-        Assert.assertTrue(count1 == 0 || count2 == 0);
+        // in key_shared mode if no message key is set, both consumers should receive messages
+        Awaitility.await()
+                .untilAsserted(() -> {
+                    Message<byte[]> message = consumer1.receive(1, TimeUnit.SECONDS);
+                    assertNotNull(message);
+                    consumer1.acknowledge(message);
+                });
+
+        Awaitility.await()
+                .untilAsserted(() -> {
+                    Message<byte[]> message = consumer2.receive(1, TimeUnit.SECONDS);
+                    assertNotNull(message);
+                    consumer2.acknowledge(message);
+                });
 
         consumer1.close();
         consumer2.close();
@@ -149,19 +143,15 @@ public class PerformanceProducerTest extends MockedPulsarServiceBaseTest {
         Awaitility.await()
                 .untilAsserted(() -> {
                     Message<byte[]> message = newConsumer1.receive(1, TimeUnit.SECONDS);
-                    if (message != null) {
-                        newConsumer1.acknowledge(message);
-                    }
                     assertNotNull(message);
+                    newConsumer1.acknowledge(message);
                 });
 
         Awaitility.await()
                 .untilAsserted(() -> {
                     Message<byte[]> message = newConsumer2.receive(1, TimeUnit.SECONDS);
-                    if (message != null) {
-                        newConsumer2.acknowledge(message);
-                    }
                     assertNotNull(message);
+                    newConsumer2.acknowledge(message);
                 });
 
         thread2.interrupt();
@@ -227,20 +217,20 @@ public class PerformanceProducerTest extends MockedPulsarServiceBaseTest {
         String topic = testTopic + UUID.randomUUID().toString();
         String args = String.format(argString, topic, pulsar.getBrokerServiceUrl(), pulsar.getWebServiceAddress());
         Consumer<byte[]> consumer = pulsarClient.newConsumer().topic(topic).subscriptionName("sub")
-                .subscriptionType(SubscriptionType.Shared).subscribe();
-        Thread thread = new Thread(() -> {
+                .subscriptionType(SubscriptionType.Key_Shared).subscribe();
+        new Thread(() -> {
             try {
                 PerformanceProducer producer = new PerformanceProducer();
                 producer.run(args.split(" "));
             } catch (Exception e) {
                 log.error("Failed to start perf producer");
             }
-        });
-        thread.start();
-        Message<byte[]> message = consumer.receive(15, TimeUnit.SECONDS);
-        assertNotNull(message);
-        thread.interrupt();
-        thread.join();
+        }).start();
+        Awaitility.await()
+                .untilAsserted(() -> {
+                    Message<byte[]> message = consumer.receive(3, TimeUnit.SECONDS);
+                    assertNotNull(message);
+                });
         consumer.close();
     }
 
